<?php
##### rrshared_innermodmenu3.php (vv1) - An Include file installed by many of by Robin Richmond's mods
/* (The numeric suffix in the Include filename and in function names are version numbers.)
The primary purpose of this Include file is to define an "Inner Mod Menu" in a TNG program.
An Inner Mod Menu is a right-floated drop-down menu in the standard TNG Inner Menu.
Inner Mod Menus contain links that supply information about the mod that created them,
giving the user handy information about the mod and a handy link to the mod options editor.
This file is installed by Mod Manager as a "protected" file, meaning that
1. If it already exists when a mod is installed, it will not generate an installation error,
   and will not be overwritten (even if it is different than the installed file), and
2. When a mod is uninstalled, it will not be removed.

See https://tng.lythgoes.net/wiki/index.php?title=Mod_Settings_Blocks to learn what 
Inner Mod Menus contain and what they look like.

This Include file contains two PHP functions that are explicitly called by PHP programs
(that have been modified by a given mod), and supporting inline css and script blocks.
The PHP functions are
1. rrNewTab3 - Places an asterisk after the label of a tab in the standard TNG tab menu.
       It is used in tabs that are created by a mod - generally to invoke a program
       that is installed by the mod. Hovering over the asterisk pops up the name of 
       the mod that created the tab.
       (This function is not related to Inner Mod Menus. It is defined in this Include
       file only so that the mod doesn't have to invoke a separate Include file for it.
       (See also rrnewtab3.php, which is used in programs that create new tabs but
       do not define an Inner Mod Menu.)
2. rrBuildInnerModMenu3 - Defines an Inner Mod Menu. Inner Mod Menus are placed in programs
      that are "significantly" modified by the mod. (They are not, for example, placed in
      programs in which the mod only defines new Tab or Inner Menu links.) Inner Mod Menus
      must be visible only to TNG Admins. They are used in most of my Admin mods. They
      can also be placed in end-user programs that are changed quite dramatically and/or
      have several mod options, especially if that end user program is also used in an 
      Admin context. Possible uses in end-user programs include:
      - getperson.php, by Regroup Person Profile and Regroup Person-Date Place, which
        change the Person Profile significantly and have several inter-related mod options.
      - placesearch.php, which is changed dramatically by Placesearch-More Info, and is
        invoked from end-user and admin programs.

The CSS and script blocks in this file are not defined as external .css and .js files
because
1. More than one mod can require this code in the same TNG program, and
2. It is a lot easier to for a program to have multiple instances of the PHP command
   include_once than to make sure that .css and .js files are attached to a page
   only once.

*************** Revision History **************
v3 1 Dec 2019
- (Of course) updated the version number suffixes in the filename and function names.
- The Inner Menu now can link to any content that describes the changes made by the mod.
  The previous version only supported links to a TNG help file.
  (A link to "content that describes the changed made by the mod" may seem redundant
  with a link to the Wiki article, but the intent to link directly to relevant content
  in TNG help files, or, now, directly to relevant content in the Wiki article.
- Fixes a problem in rrNewTab*, where the asterisk was explicitly made white, and was
  not visible in certain TNG templates until the user moused over the tab. Now it is 
  the same color as the tab's label. It is also part of the tab's hyperlinked label.
  That is, when a user mouses over the asterisk (but not the label's text) the mod name
  pops up, but when the user clicks any part of the label, including the asterisk, the
  tab's hyperlink is invoked.
- The rrBuildInnerModMenu* parameter that says whether there is a Wiki article section
  describing the mod options can now be a mod article heading other than "Mod Options".
- Added a parameter to rrBuildInnerModMenu* that specifies the wiki article name, in
  case the article name is different from the mod name.
- Removed the explicit onlick event handler for the menu's X button. It is not needed
  since the menu is closed by any click on the menu that is not directly on a hyperlink.
v2 1 Oct 2019
- Added version number suffixes to the function names.
- The X that closes in Inner Mod Menu is no longer in a <a> element. A style class on
  the <img> tag makes the cursor a hyperlink pointer, and since clicking on any part of
  the menu (other than a hyperlink) now closes it, the X doesn't need its own hyerlink.
- Corrected the Inner Menu IDs to use dash rather than underscore.
- Tests ($('#adm-innermenu').length) rather than just ($('#adm-innermenu')) to determine
  which inner menu ID to use. (They are different in admin vs end-user programs.)
- Changes the size of the Wiki icon and puts a border on it to make it look more
  like a hyperlink.  (I'm not crazy about this approach, but I felt that I needed
  to do something. */
?>
<script>
/////////////////////////////////////////////
//document.ready function for programs that have an Inner Mod Menu.
//The PHP code in rrBuildInnerModMenu* places an Inner Mod Menu in a div with
//class='rrinnermodmenu'.  This function merges all of the program's Inner Mod Menus
//into one menu that contains all links from all of the original Inner Mod Menus.
//The consoldated Inner Mod Menu is defined in a div with id=innermodmenu.
///////////////////////////////////////////
$(function () {
	var rrModMenuOccurrences = $('.rrInnerModMenu').length;
	if (!rrModMenuOccurrences) return; //***************Early exit - no menus

	//Build the consolidated InnerModMenu contents from the InnerModMenus of each mod. 
	var rrModMenuHTML = '';
	var rrModCount = 0;
	$('.rrInnerModMenu').each (function() {
		rrModMenuHTML += $(this).html();
		++rrModCount;
		$(this).remove();
	})

	//Add the InnerModMenu wrapper to the Inner menu.
	var rrInnerMenuID;
	if ($('#adm-innermenu').length) rrInnerMenuID = '#adm-innermenu';
	else rrInnerMenuID = '#pub-innermenu';
	var html = "<div id='rrInnerModMenu'>";
	html += "<a href='#' class='lightlink' onclick='return false;'>Mod Information</a>";
	html += "<div id='rrInnerModDrop' class='slidedown'>";
	html += "<img src='img/tng_close.gif' class='rrInnerModMenuX' border='0' alt=''>";
	html += "</div></div>";
	$(rrInnerMenuID).append(html);

	//Add the InnerModMenu contents to the wrapper
	$('#rrInnerModDrop').append(rrModMenuHTML);

	//Define the height of the InnerModMenu, based on the count of lines (<br> tags) in the content.
	//Plus 1 count for each mod, based on the height of the Wiki logo
	var rrNumLines = rrModMenuHTML.split('<br>').length + rrModCount;
	$('#rrInnerModDrop').css('height',(rrNumLines*1.2)+'em');

	//Onclick function to toggle the display of the InnerModMenu when the user clicks
	//on any part of the menu that is not a hyperlink. This is the only way to
	//open or close the menu. When the menu is closed, only the menu's label is visible.
	//When the menu is open, its X does not need to explictly close the menu, since
	//the X is simply part of the menu. 
	$('#rrInnerModMenu').click(function () {
		$('#rrInnerModDrop').slideToggle(200);
		return false;
	});
}); //end document.ready function

//////////////////////////////////////////////////
//Onclick event handler to open a TNG Wiki page as a popup.
//It is used both for the Wiki article hyperlink and the link to the Mod Options
//section of the Wiki page. In both cases the article argument in the PHP code
//is derived from the Mod Name
function rrInnerModOpenWikiPage3(article, anchor='') {
	if (anchor) anchor="#"+anchor;
	newwindow = window.open('https://tng.lythgoes.net/wiki/index.php?title='+article+anchor, 'wikiwindow', 'height=600, width=780, top=50, left=100, resizable=yes, scrollbars=yes, status=yes');
	newwindow.focus();
	return false;
} //end function rrInnerMenuOpenWikiPage

//////////////////////////////////////////////////
//This is the onclick handler for the Inner Mod Menu link that invokes the mod'sOpen
//Mod Options editor (which is necessarily a TNG Admin>>Setup program).
//The editor is invoked as a popup page.
//For the two Admin>>Setup programs, that have subforms, a default subform name is
//used if $subform parameter to rrBuildInnerModMenu3* was not specified.
function rrInnerModOpenConfigPage3(configprogram, subform='', modid='') {
	if (configprogram.indexOf('.php') == -1)
		configprogram += '.php';
	if (!subform) {
		if (configprogram == 'admin_genconfig.php') subform = 'robin';
		else if (configprogram == 'admin_pedconfig.php') subform = 'peddesc';
	}
	newwindow = window.open(configprogram+"?open="+subform+'&mod='+modid, 'optionswindow', 'height=600, width=780, top=50, left=100, resizable=yes, scrollbars=yes, status=yes');
	newwindow.focus();
	return false;
} //end function rrInnerModOpenConfigPage

</script>

<style>
/*********** Styles for Inner Mod Menus *******************/
#rrInnerModMenu {float:right; padding-right:8px; position:relative;}
.rrInnerModMenuX {padding-left:15px; float:right; cursor:pointer;}
#rrInnerModDrop {z-index:200; color:orange;
	position:absolute; top:30px; left:-168px;
	padding:2px 4px;
	width:240px; height:4em; margin-right:15px;
	border-radius:5px;
	line-height:1.4em;
	display:none;
}
#rrInnerModDrop>a {text-decoration:none;}
#rrInnerModDrop>a:hover {text-decoration:underline;}
.rrInnerModWiki {cursor:pointer; width:20px; height:20px; }
.rrInnerModWiki:hover {border-bottom:thin solid #ccc;}
.rrInnerModNotLink{color:black;}
/* The New Tab pop-up */
.rrNewTab {
	float:right;
	position: relative;
}
/* Format the the "Installed by {mod name}" popup */
.rrNewTab .rrNewModname {
	position: absolute;
	display:none;
	white-space:nowrap;
	color:black;
	top:-2em;
	left:-4em;
	background: #ffffff;
	padding: 2px;
	border:thin solid grey;
	z-index: 1001;
}
/* Display the "Installed by {mod name}" popup */
.rrNewTab>div:hover>.rrNewModname{
	display:block;
}
</style>

<?php
########################################################################
######################## PHP Functions #################################
########################################################################
# This function is called when new tabs are added to the end-user tab menu.
# It places an asterisk after the tab label to flag that this is a new tab.
# The asterisk's title text is the name of the mod that defined that new tab.
function rrNewTab3($modname) {
	$html = "<div class='rrNewTab'><div>&nbsp;*<div
	class='rrNewModname'>Installed by $modname</div></div></div>";
	return $html;
} #end function rrNewTab*

########################################################################
/* Function to define the HTML code for Inner Mod Menu links.
Since a program can be modified by more than one mod, more than one call to the
function can be inserted into a program, and the Inner Mod Menu can contain links
for more than one mod.
Input:
  $modname - (I'll make you guess)
  $haswikioptions - Truthy if there is a section in the mod's Wiki article that
     describes the mod's options. If its length is >1, then it is presumed to be
     the heading in the article. Otherwise, the heading defaults to 'Mod_Options'
  $configprogram - The Admin>>Setup program through which this mod's options can be
     edited. If non-empty, the function creates a link to the mod's options's fields.
  $subform - The subform where this mods options are placed in the configuration program.
       Can be empty. If $configprogram is empty, this option is ignored. Otherwise
       it defaults to a specific subform.
  $helppage - The URL (possibly including an anchor) of a page that describes the mod's
      changes. Frequently omitted, since the Wiki page certainly contains such content
      somewhere.  But this URL is expected to link directly to explicit help text,
      which is likely to be in the program's help file
  $wikiname - The mod wiki article name, if it is different from the mod name. With
      my mods, this is not likely to be needed, but you never know.
Return value:
  The HTML code for menu, including the drop-down link, in a divwith class='innermodmenu'.
 
Note that the hyperlink text that this function generates (e.g. "Mod Information",
"Mod Options Help", "Change Mod Options", and "What has Changed"), is hard-coded in
English. To support multiple languages, either:
- This Include file would have to be installed by a stand-alone mod (which would
  necessarly be required by all mods that use it), or
- Each mod that uses the Include file would have to (redundantly) define the 
  necessary language strings.
*/
function rrBuildInnerModMenu3($modname, $haswikioptions=0, $configprogram='', $subform='', $helppage='') {
	global $admtext, $helplang;
	#Always use the mod name as a heading. Define it as a link so that it has the right styling.
	#on the same line as the heading, add a Wiki icon that pops up the mod's Wiki article.
	$html = "<div class='rrInnerModMenu' style='display:none;'>";
	$html .= "<span class='nw rrInnerModNotLink'><b style='font-size:110%;'>$modname</b>&nbsp;";
	$html .= "&nbsp;&nbsp;<a href='#' onclick=\"return rrInnerModOpenWikiPage3('$modname');\">";
	$html .= "<img src='img/rrshared_wikilogo.gif' border='2' style='width:16px;height:16px;' class='rrInnerModWiki'></a><br>";

	if ($haswikioptions) {
		#Add a link that pops up the mod's wiki article, positioned to a Mod Options section.
		if (strlen($haswikioptions)>1)
			$anchor = str_replace(' ', '_', $haswikioptions);
		else $anchor = "Mod_Options";
		$html .= "&nbsp;&nbsp;&nbsp;<a href='#' class='nw' onclick=\"return rrInnerModOpenWikiPage3('$modname','$anchor');\"><b>Mod Options Help</b></a><br>";
	}
	if ($configprogram) {
		#Add a link to the mod options editor. Allow the required 'admin_' prefix and
		#'php' file extension to be omitted.
		if (substr($configprogram,0,6) != 'admin_') $configprogram = "admin_$configprogram";
		if (substr($configprogram, -4) != '.php') $configprogram .= '.php';
		$modid = str_replace(' ', '', $modname);
		$html .= "&nbsp;&nbsp;&nbsp;<a href='#' onclick=\"return rrInnerModOpenConfigPage3('$configprogram', '$subform', '$modid');\" class='nw'><b>Change Mod Options</b></a><br>";
	}

	if ($helppage) {
		#Define a link to the relevant help page (which was originally, but is no longer
		#required to be a TNG help page.
		#The path to a TNG help page can be omitted, in which case, the URL in $helppage
		#has no path (no slash).
		if (strpos($helppage,'/') === false)
			$helppage = "$helplang/$helppage";
		$html .= "&nbsp;&nbsp;&nbsp;<a href='#' onclick=\"return openHelp('$helppage');\"><b>What has changed</b></a><br>";
	}
	return $html . "</div>";
} #end function rrBuildInnerModMenu
?>
